<?php
namespace Ak\Db\Adapter\Driver\Dbal;

use Doctrine\DBAL\Connection;
use Laminas\Db\Adapter\Driver\ConnectionInterface;
use Laminas\Db\Adapter\Driver\DriverInterface;
use Laminas\Db\Adapter\Driver\Pdo\Feature\SqliteRowCounter;
use Laminas\Db\Adapter\Driver\Pdo\Result;
use Laminas\Db\Adapter\Driver\ResultInterface;
use Laminas\Db\Adapter\Driver\StatementInterface;

/**
 * Class DbalDriverTrait
 *
 * @package Ak\Db\Adapter\Driver\Dbal
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
trait DbalDriverTrait
{
    /** @var ConnectionInterface|Connection */
    private $connection;

    /**
     * Get database platform name
     *
     * @param string $nameFormat
     * @return string
     */
    public function getDatabasePlatformName(/** @noinspection PhpUnusedParameterInspection */ $nameFormat = self::NAME_FORMAT_CAMELCASE)
    {
        return 'ne koristi se nigdje';
    }

    /**
     * @return ConnectionInterface|Connection
     */
    public function getConnection(): ConnectionInterface
    {
        return $this->connection;
    }

    /**
     * @param ConnectionInterface $connection
     */
    public function setConnection(ConnectionInterface $connection): void
    {
        $this->connection = $connection;
    }

    /**
     * @return string
     */
    public function getPrepareType()
    {
        return DriverInterface::PARAMETERIZATION_NAMED;
    }

    /**
     * @param string $name
     * @param string|null $type
     * @return string
     */
    public function formatParameterName($name, $type = null)
    {
        if ($type === null && ! is_numeric($name) || $type == DriverInterface::PARAMETERIZATION_NAMED) {
            $name = ltrim($name, ':');
            // @see https://bugs.php.net/bug.php?id=43130
            if (preg_match('/[^a-zA-Z0-9_]/', $name)) {
                throw new \RuntimeException(sprintf(
                    'The PDO param %s contains invalid characters.'
                    . ' Only alphabetic characters, digits, and underscores (_)'
                    . ' are allowed.',
                    $name
                ));
            }
            return ':' . $name;
        }

        return '?';
    }

    /**
     * Check environment
     */
    public function checkEnvironment()
    {
        if (!extension_loaded('PDO')) {
            throw new \RuntimeException(
                'The PDO extension is required for this adapter but the extension is not loaded'
            );
        }
    }

    /**
     * Get last generated value
     *
     * @return mixed
     */
    public function getLastGeneratedValue()
    {
        return $this->connection->lastInsertId();
    }

    /**
     * Create statement
     *
     * @param string|resource $sqlOrResource
     * @return StatementInterface
     */
    public function createStatement($sqlOrResource = null)
    {
        if (!$this->connection->isConnected()) {
            $this->connection->connect();
        }

        /** @var \PDO $pdo */
        $pdo = $this->connection->getResource();
        $pdoStatement = $sqlOrResource ? $pdo->prepare($sqlOrResource) : null;

        $statement = new Statement();
        /** @noinspection PhpParamsInspection */
        $statement->setPdoDriver($this);
        $statement->initialize($pdo);

        if (is_string($sqlOrResource)) {
            $statement->setSql($sqlOrResource);
        }

        if ($pdoStatement) {
            $statement->setResource($pdoStatement);
        }

        return $statement;
    }

    /**
     * Create result
     *
     * @param \PDOStatement $resource
     * @param null $context
     * @return ResultInterface
     */
    public function createResult($resource, $context = null)
    {
        $result = new Result();
        $rowCount = null;

        if ('pdo_sqlite' === $this->getName()) {
            $rowCounter = new SqliteRowCounter();
            /** @noinspection PhpParamsInspection */
            $rowCounter->setDriver($this);
            $rowCount = $rowCounter->getRowCountClosure($context);
        }

        $result->initialize($resource, $this->getLastGeneratedValue(), $rowCount);

        return $result;
    }
}
