<?php
namespace Ak\Db\TableGateway;

use Ak\Db\Adapter\Driver\Dbal\Connection;
use Ak\Db\Adapter\Driver\DriverInterface;
use Interop\Container\ContainerInterface;
use Laminas\Db\Adapter\AdapterInterface;
use Laminas\Db\Sql\TableIdentifier;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;

/**
 * Stvara nove instance TableGatewaya.
 *
 * @package Ak\Db\TableGateway
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class TableGatewayAbstractFactory implements AbstractFactoryInterface
{
    /**
     * Can the factory create an instance for the service?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        $tables = $container->get('config')['db']['tables'] ?? [];
        return isset($tables[$requestedName]) && is_array($tables[$requestedName]);
    }

    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return TableGatewayInterface
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     * @throws \Doctrine\DBAL\DBALException
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $options ?? $container->get('config')['db']['tables'][$requestedName];

        if (!isset($config['name'])) {
            throw new \RuntimeException("Nedostaje parametar 'name'.");
        }

        /** @var AdapterInterface $adapter */
        $adapter = isset($config['adapter'])
            ? $container->get($config['adapter'])
            : $container->get('db');

        $gatewayClass = isset($config['gateway'])
            ? $config['gateway']
            : TableGateway::class;

        if (!class_exists($gatewayClass)) {
            throw new \RuntimeException(sprintf("Ne mogu pronaći TableGateway klasu '%s'.", $gatewayClass));
        }

        $gateway = new $gatewayClass(new TableIdentifier($config['name'], $config['schema'] ?? null), $adapter);

        if ($gateway instanceof TableGatewayInterface) {
            $driver = $adapter->getDriver();

            if ($driver instanceof DriverInterface) {
                /** @var Connection $conn */
                $conn = $driver->getConnection();
                $schemaManager = $conn->getSchemaManager();

                $tableName = $gateway->getTable();

                if ($tableName instanceof TableIdentifier) {
                    $tableName = $tableName->getTable();
                }

                if (!in_array($tableName, $schemaManager->listTableNames())) {
                    $fromSchema = $schemaManager->createSchema();
                    $toSchema = clone $fromSchema;

                    $table = $toSchema->createTable($tableName);
                    $gateway->createSchema($table);

                    if (count($table->getColumns()) > 0) {
                        $sql = $fromSchema->getMigrateToSql($toSchema, $conn->getDatabasePlatform());

                        foreach ($sql as $line) {
                            $conn->exec($line);
                        }
                    }
                }
            }
        }

        return $gateway;
    }
}
