<?php
namespace AkTest\Db\Adapter;

use Ak\Db\Adapter\AdapterConfigFactory;
use Ak\Db\Exception\ConfigurationException;
use Ak\Test\AbstractTestCase;
use AkTest\Db\TestConfigProvider;
use Laminas\Db\Adapter\AdapterInterface;

/**
 * Class AdapterConfigFactoryTest
 *
 * @package AkTest\Db\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class AdapterConfigFactoryTest extends AbstractTestCase
{
    public $configProviders = [
        TestConfigProvider::class,
    ];

    public function testWillGetDefaultConfig()
    {
        $container = $this->getContainer([
            'db' => [
                'driver' => 'db.driver.sqlite',
            ],
        ]);

        $factory = new AdapterConfigFactory();
        $config = $factory($container, 'db');

        $this->assertArrayHasKey('driver', $config);
        $this->assertEquals('db.driver.sqlite', $config['driver']);
        $this->assertEquals($config, $factory($container, AdapterInterface::class));
    }

    /**
     * @expectedException \Ak\Db\Exception\ConfigurationException
     */
    public function testWillThrowExceptionIfConfigDoesntHaveADriverKey()
    {
        $container = $this->getContainer([
            'db' => [
                'default_adapter' => 'no_driver',

                'adapters' => [
                    'no_driver' => [],
                ],
            ],
        ]);

        $factory = new AdapterConfigFactory();
        $factory($container, 'db');
    }

    /**
     * @expectedException \Ak\Db\Exception\ConfigurationException
     */
    public function testWillThrowExceptionForInvalidDriver()
    {
        $container = $this->getContainer([
            'db' => [
                'default_adapter' => 'invalid_driver',

                'adapters' => [
                    'invalid_driver' => [
                        'driver' => 'invalid',
                    ],
                ],
            ],
        ]);

        $factory = new AdapterConfigFactory();
        $factory($container, 'db');
    }

    public function testWillThrowExceptionIfDefaultAdapterDoesntHaveConfig()
    {
        $container = $this->getContainer([
            'db' => [
                'default_adapter' => 'invalid_adapter',
            ],
        ]);

        $factory = new AdapterConfigFactory();

        try {
            $factory($container, 'db');
        } catch (ConfigurationException $e) {
            $this->assertContains('invalid_adapter', $e->getMessage());
        }
    }

    public function testIfSetWillReturnDefaultAdapterConfig()
    {
        $adapterConfig = [
            'driver' => 'db.driver.sqlite',
            'hostname' => 'other',
        ];

        $container = $this->getContainer([
            'db' => [
                'default_adapter' => 'some_adapter',

                'adapters' => [
                    'some_adapter' => $adapterConfig,
                ],
            ],
        ]);

        $factory = new AdapterConfigFactory();
        $config = $factory($container, 'db');

        $this->assertEquals($adapterConfig, $config);
    }

    public function testWillThrowExceptionForUnknownAdapter()
    {
        $container = $this->getContainer();

        $factory = new AdapterConfigFactory();

        try {
            $factory($container, 'invalid_adapter');
        } catch (ConfigurationException $e) {
            $this->assertContains('invalid_adapter', $e->getMessage());
        }
    }

    public function testWillReturnConfigForAdapter()
    {
        $adapterConfig = [
            'driver' => 'db.driver.sqlite',
            'hostname' => 'other',
        ];

        $container = $this->getContainer([
            'db' => [
                'adapters' => [
                    'some_adapter' => $adapterConfig,
                ],
            ],
        ]);

        $factory = new AdapterConfigFactory();
        $config = $factory($container, 'some_adapter');

        $this->assertEquals($adapterConfig, $config);
    }
}
