<?php
namespace AkTest\Db\TableGateway;

use Ak\Db\TableGateway\TableGateway;
use Ak\Test\AbstractTestCase;
use Ak\Test\DbTestTrait;
use AkTest\Db\TestConfigProvider;
use Doctrine\DBAL\Schema\Schema;
use PHPUnit\DbUnit\DataSet\ArrayDataSet;
use Laminas\Db\Adapter\AdapterInterface;

class TableGatewayTest extends AbstractTestCase
{
    use DbTestTrait;

    public $configProviders = [
        TestConfigProvider::class,
    ];

    /**
     * @param Schema $schema
     * @throws \Doctrine\DBAL\DBALException
     */
    public function createSchema(Schema $schema): void
    {
        if (!$schema->hasTable('test_table')) {
            $table = $schema->createTable('test_table');
            $gateway = new MockTableGateway('test_table', $this->lastContainer->get('db'));
            $gateway->createSchema($table);
        }
    }

    protected function getDataSet()
    {
        return new ArrayDataSet([
            'test_table' => [
                ['id' => 1, 'name' => 'One'],
                ['id' => 2, 'name' => 'Two'],
            ],
        ]);
    }

    public function testCanReadRows()
    {
        $container = $this->lastContainer;
        $adapter = $container->get('db');

        $table = new TableGateway('test_table', $adapter);

        $rows = $table->select();
        $this->assertCount(2, $rows);

        $this->assertEquals([
            ['id' => 1, 'name' => 'One'],
            ['id' => 2, 'name' => 'Two'],
        ], $rows->toArray());
    }

    public function testCanSelectRowsWithWhere()
    {
        $container = $this->lastContainer;
        $adapter = $container->get('db');

        $table = new TableGateway('test_table', $adapter);

        $rows = $table->select(['id' => 1])->toArray();
        $this->assertCount(1, $rows);

        $this->assertEquals([['id' => 1, 'name' => 'One']], $rows);

        $other = $table->select(['name' => 'One'])->toArray();
        $this->assertEquals($rows, $other);
    }

    public function testCanInsertRows()
    {
        $container = $this->lastContainer;
        /** @var AdapterInterface $adapter */
        $adapter = $container->get('db');

        $table = new TableGateway('test_table', $adapter);
        $affectedRows = $table->insert([
            'name' => 'Three'
        ]);

        $this->assertEquals(1, $affectedRows);

        $rows = $table->select(['name' => 'Three'])->toArray();

        $this->assertCount(1, $rows);
        $this->assertEquals([['id' => 3, 'name' => 'Three']], $rows);
    }

    public function testCanUpdateRows()
    {
        $container = $this->lastContainer;
        /** @var AdapterInterface $adapter */
        $adapter = $container->get('db');

        $table = new TableGateway('test_table', $adapter);

        $affectedRows = $table->update([
            'name' => 'NotOne'
        ], [
            'id' => 1,
        ]);

        $this->assertEquals(1, $affectedRows);

        $rows = $table->select(['id' => 1])->toArray();

        $this->assertCount(1, $rows);
        $this->assertEquals([['id' => 1, 'name' => 'NotOne']], $rows);
    }
}
